/*
 * MicroProject.c
 * 
 * MCP4725 I2C DAC Simple Driver Example
 *
 * Created: 6. 5. 2023 13:55:04
 * Author : greif
 */ 

// Includes
#include <stdio.h>
#include <avr/io.h>
#include "uart.h" // UART for SerialPlot output

#define DAC_ADDR 0x60 // MCP4725 address - if not set here, default address is 0x60
#include "mcp4725.h" // MCP4725 Simple Driver


FILE uart_stream = FDEV_SETUP_STREAM(uart_putc, uart_getc, _FDEV_SETUP_RW); // Link I/O to UART



// ADC initialization
void adc_init(void){
	ADMUX = (0<<REFS1)|(1<<REFS0); // AVCC - set the ADC reference voltage
	ADCSRA = (1<<ADEN) // "Turn ON" the ADC
	|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0); // Set the prescaler - fOsc/128, fADC = 125kHz, conversion duration < ako 0,1ms, align right
}



// Read ADC channel
uint16_t adc_read(uint8_t a_pin){
	a_pin &= 0x07;
	ADMUX = (ADMUX & 0xF8)|a_pin; // Set the channel
	
	ADCSRA |= (1<<ADSC); // Start the conversion
	while(ADCSRA & (1<<ADSC)); // Wait until conversion done
	
	return (ADC);
}




int main(void)
{
	DACInit(); // Initialize the MCP4725 12b DAC on the I2C bus
	
	adc_init(); // Initialize the on-board 10b ADC
	
	// Initialize UART communication
	uart_init();
	stdout = stdin = &uart_stream;
	printf("Ready to start...\n\n");

	setDACValue(2048, 1); // Write a 12b (0-4095) value to DAC register and EEPROM
	
	// Gradually increase MCP4725 output, while displaying measured voltage and MCP4725 register/EEPROM readings
	uint16_t i = 0;
    while (1) 
    {
		uint16_t adcValue = adc_read(0); // Get ADC reading from the MCP4725 output pin
		uint16_t eeprom = readEEPROM(); // Read current EEPROM value from MCP4725 (persistent)
		uint16_t current = readDACValue(); // Read current DAC register value from MCP_4725 (temporary)
		
		// Reset counter if out of MCP4725 12b range
 		if(i > DAC_MAX){
			 i = DAC_MIN;
		}
		
		setDACValue(i, 0); // Write a 12b (0-4095) value to DAC register - without writing the EEPROM
		i+=10; // Increase the counter
		
		printf("%u %u %u\r\n", adcValue, eeprom, current); // Send measured values to UART (SerialPlot)
    }
	
	return 0;
}
